import os
import threading
from dataclasses import dataclass

import pytest

from sqlink import Entity, Define as d, Dao, DaoFunc, Sql
from sqlink.database import DB, DBFunc


@Entity
@dataclass
class Student:
    name: str
    student_id: int = d.auto_primary_key


@Dao(Student)
class DaoStudent(DaoFunc):
    @Sql("select * from __;", fetch_type=Student)
    def select_all(self):
        pass


@DB
class Database(DBFunc):
    dao_student = DaoStudent()


# ====================================================================================================================
def generate_students(start, end):
    return [Student(name=f'李华{i}', student_id=i) for i in range(start, end)]


def insert_data(db: Database, num=100):
    students = generate_students(0, num)
    db.dao_student.insert(students)
    db.commit()
    return students


def get_test_db_path(db_name):
    func_path = __file__
    db_path = os.path.join(os.path.dirname(func_path), f"test_db\\{db_name}.db")
    if not os.path.exists(os.path.dirname(db_path)):
        os.makedirs(os.path.dirname(db_path))
    return db_path


def get_db(db_name):
    db = Database()
    db.connect(f"sqlite://{get_test_db_path(db_name)}", check_same_thread=False)
    return db


# ====================================================================================================================
def test_multiple_init_and_use():
    db_name = 'same_thread'
    db_path = get_test_db_path(db_name)
    if os.path.exists(db_path):
        os.remove(db_path)
    db = get_db(db_name)
    db.create_tables()
    students = insert_data(db, 100)

    def query():
        temp_db = get_db(db_name)
        result = temp_db.dao_student.select_all()
        assert result == students

    for _ in range(10):
        query()


def test_multiple_init_in_thread():
    db_name = 'multiple_thread'
    db_path = get_test_db_path(db_name)
    if os.path.exists(db_path):
        os.remove(db_path)
    db = get_db(db_name)
    db.create_tables()
    students = insert_data(db, 100)

    def query():
        temp_db = get_db(db_name)
        result = temp_db.dao_student.select_all()
        assert result == students

    for _ in range(10):
        task = threading.Thread(target=query)
        task.start()


if __name__ == '__main__':
    pytest.main(["-vv", "--capture=no", __file__])



import inspect
import os


# class MysqlDatabase(MetaDB):
#     def __init__(self):
#         super().__init__()
#         self.update_db_type("mysql")
#
#     def connect(self):
#         connection = pymysql.connect(host='localhost',
#                                      user='root',
#                                      password='123456')
#         self.update_connection(connection)
#         # 删除数据库（如果存在）
#         self.cursor.execute("DROP DATABASE IF EXISTS test_database")
#         self.cursor.execute("CREATE DATABASE IF NOT EXISTS test_database")
#         # 选择使用新创建的数据库
#         self.cursor.execute("USE test_database")


def use_db(database, sql: str = None):
    def decorator(deco_func):
        def wrapper(*args, **kwargs):
            db = database()
            func_path = inspect.getfile(deco_func)
            # 提取文件名（不含扩展名）
            file_name = os.path.splitext(os.path.basename(func_path))[0]
            db_path = os.path.join(os.path.dirname(func_path), f"test_db\\{file_name}.db")
            if not os.path.exists(os.path.dirname(db_path)):
                os.makedirs(os.path.dirname(db_path))

            if os.path.exists(db_path):
                os.remove(db_path)
            db.connect(f"sqlite://{db_path}")
            if sql:
                db.execute(sql)
            db.create_tables()
            deco_func(*args, **kwargs, db=db)
            db.disconnect()
            if os.path.exists(db_path):
                os.remove(db_path)

        return wrapper

    return decorator



# -*- coding:utf-8 -*-
from dataclasses import dataclass

import pytest

from sqlink.entity import _parse_attr_value, _parse_defines, Entity, Define as d, _Define


@Entity
@dataclass
class Student:
    attr_int: int
    attr_float: float
    attr_bool: bool
    attr_str: str

    attr_tuple: tuple
    attr_list = list
    attr_dict = dict
    attr_set = set

    attr_int_default: int = 1
    attr_float_default: float = 1.0
    attr_bool_default: bool = True
    attr_str_default: str = '1'

    attr_1d: str = d.not_null
    attr_2d: str = d.not_null, d.unique
    attr_3d: str = d.not_null, d.unique, d.primary_key

    attr_str_and_d: str = '1', d.not_null, d.datatype("varchar(200)")
    attr_int_and_d: int = 1, d.not_null
    attr_float_and_d: float = 1.0, d.not_null
    attr_bool_and_d: bool = True, d.not_null

    attr_str_and_2d: str = '1', d.not_null, d.unique, d.datatype("varchar(200)")
    attr_int_and_2d: int = 1, d.not_null, d.unique
    attr_float_and_2d: float = 1.0, d.not_null, d.unique
    attr_bool_and_2d: bool = True, d.not_null, d.unique

    id: int = d.auto_primary_key

    attr_tuple_c: tuple = d.ignore
    attr_list_c: list = d.ignore
    attr_dict_c: dict = d.ignore
    attr_set_c: set = d.ignore

    attr_tuple_c_and_v: tuple = d.ignore, (1,)
    attr_tuple2_c_and_v: tuple = d.ignore, 1, 2
    attr_list_c_and_v: list = d.ignore, [1]
    attr_dict_c_and_v: dict = d.ignore, {'1': 1}
    attr_set_c_and_v: set = d.ignore, {1}


# ====================================================================================================================
def test_parse_1value():
    assert Student.attr_int_default == _parse_attr_value(Student.attr_int_default)
    assert Student.attr_float_default == _parse_attr_value(Student.attr_float_default)
    assert Student.attr_bool_default == _parse_attr_value(Student.attr_bool_default)
    assert Student.attr_str_default == _parse_attr_value(Student.attr_str_default)


def test_parse_1v_from_1c():
    assert Student.attr_int_default == _parse_attr_value(Student.attr_int_and_d)
    assert Student.attr_float_default == _parse_attr_value(Student.attr_float_and_d)
    assert Student.attr_bool_default == _parse_attr_value(Student.attr_bool_and_d)
    assert Student.attr_str_default == _parse_attr_value(Student.attr_str_and_d)


def test_parse_1v_from_2c():
    assert Student.attr_int_default == _parse_attr_value(Student.attr_int_and_2d)
    assert Student.attr_float_default == _parse_attr_value(Student.attr_float_and_2d)
    assert Student.attr_bool_default == _parse_attr_value(Student.attr_bool_and_2d)
    assert Student.attr_str_default == _parse_attr_value(Student.attr_str_and_2d)


def test_parse_1define():
    assert isinstance(Student.attr_1d, _Define)
    assert _parse_defines(Student.attr_1d) == [d.not_null]
    assert _parse_defines(Student.id) == [d.auto_primary_key]


def test_parse_2c():
    defines = {d.not_null, d.unique}
    assert set(_parse_defines(Student.attr_2d)) == defines


def test_parse_3c():
    defines = {d.not_null, d.unique, d.primary_key}
    assert set(_parse_defines(Student.attr_3d)) == defines


def test_parse_value_from_c_no_default():
    assert _parse_attr_value(Student.attr_1d) is None
    assert _parse_attr_value(Student.attr_2d) is None
    assert _parse_attr_value(Student.attr_3d) is None


def test_parse_1c_from_1v():
    assert _parse_defines(Student.attr_str_and_d) == [d.not_null, d.datatype("varchar(200)")]
    assert _parse_defines(Student.attr_int_and_d) == [d.not_null]
    assert _parse_defines(Student.attr_bool_and_d) == [d.not_null]
    assert _parse_defines(Student.attr_float_and_d) == [d.not_null]


def test_parse_2c_from_1v():
    defines = {d.not_null, d.unique}
    assert set(_parse_defines(Student.attr_str_and_2d)) == {d.not_null, d.unique, d.datatype("varchar(200)")}
    assert set(_parse_defines(Student.attr_int_and_2d)) == defines
    assert set(_parse_defines(Student.attr_bool_and_2d)) == defines
    assert set(_parse_defines(Student.attr_float_and_2d)) == defines


def test_parse_ignore_define():
    assert _parse_defines(Student.attr_tuple_c) == [d.ignore]
    assert _parse_defines(Student.attr_list_c) == [d.ignore]
    assert _parse_defines(Student.attr_dict_c) == [d.ignore]
    assert _parse_defines(Student.attr_set_c) == [d.ignore]


def test_parse_ignore_value():
    assert _parse_attr_value(Student.attr_tuple_c_and_v) == (1,)
    assert _parse_attr_value(Student.attr_tuple2_c_and_v) == (1, 2)
    assert _parse_attr_value(Student.attr_list_c_and_v) == [1]
    assert _parse_attr_value(Student.attr_dict_c_and_v) == {'1': 1}
    assert _parse_attr_value(Student.attr_set_c_and_v) == {1}


def test_parse_ignore_value_no_default():
    assert _parse_attr_value(Student.attr_tuple_c) is None
    assert _parse_attr_value(Student.attr_list_c) is None
    assert _parse_attr_value(Student.attr_dict_c) is None
    assert _parse_attr_value(Student.attr_set_c) is None


if __name__ == '__main__':
    pytest.main(["-vv", "--capture=no", __file__])



from dataclasses import dataclass

import pytest

from sqlink import Entity, Define as d


def test_entity_check_ignore_type():
    @Entity(check_type=True)
    @dataclass
    class Student:
        name: str
        score: list

    with pytest.raises(TypeError) as error:
        Student('1', 1)
    assert "实例化" in str(error.value)
    assert "类时," in str(error.value)

    @Entity(check_type=True)
    @dataclass
    class Student2:
        name: str
        score: list = d.ignore

    Student2('1', 1)


if __name__ == '__main__':
    pytest.main(["-v", __file__, "--log-cli-level=INFO"])



# -*- coding:utf-8 -*-
from dataclasses import dataclass

import pytest

from sqlink import Entity, Define as d, Dao, DaoFunc, Sql
from sqlink.dao import DB_TYPE_SQLITE, DB_TYPE_MYSQL
from sqlink.database import DB
from test.test_env import use_db


@Entity(check_type=True)
@dataclass
class Student:
    name: str = None
    score: int = d.ignore
    address: str = d.ignore
    student_id: int = d.auto_primary_key


@Dao(entity=Student)
class DaoStudent(DaoFunc):
    @Sql("select * from __ where student_id=?;")
    def select_student(self, student_id):
        pass


@DB
class Database:
    dao_student = DaoStudent()


# ====================================================================================================================
@use_db(Database)
def test_ignore(db: Database):
    """数据类的忽略属性在建表时"""
    db.dao_student.create_table()
    db.commit()
    if db.db_type == DB_TYPE_SQLITE:
        db.dao_student.cursor.execute(f"PRAGMA table_info({Student.__name__});")
        # 获取所有列信息
        columns = db.dao_student.cursor.fetchall()
        # 提取并打印所有字段名
        columns = set(column[1] for column in columns)
    elif db.db_type == DB_TYPE_MYSQL:
        sql = """
                SELECT COLUMN_NAME 
                FROM INFORMATION_SCHEMA.COLUMNS 
                WHERE TABLE_SCHEMA = %s 
                AND TABLE_NAME = %s
                ORDER BY ORDINAL_POSITION;
                """
        db.dao_student.cursor.execute(sql, ("test_database", Student.__name__))
        columns = db.dao_student.cursor.fetchall()
        columns = set(column[0] for column in columns)

    entity_fields = set(attr_name for attr_name, attr_type in Student.__annotations__.items())
    ignore_fields = {'score', 'address'}
    entity_fields_without_ignore = set(filter(lambda item: item not in ignore_fields, entity_fields))

    assert entity_fields != columns
    assert entity_fields_without_ignore == columns


@use_db(Database)
def test_insert_ignore(db: Database):
    """数据类的忽略属性在插入记录时"""
    student = Student(name='张三', score=100, address='hit')
    db.dao_student.insert(student)
    db.commit()


if __name__ == '__main__':
    pytest.main(["-vv", "--capture=no", __file__])



# -*- coding:utf-8 -*-
import pytest

from sqlink import Entity, Define as d, Dao, DaoFunc
from sqlink.database import DB, DBFunc
from test.test_env import use_db


@Entity
class Student_Comment:
    name: str = d.comment('学生姓名')
    contact_info: float = d.comment('联系方式')
    time: str = d.comment('时间')
    abstract: str = d.comment('摘要')
    student_id: int = d.auto_primary_key, d.comment('学生id')


@Dao(Student_Comment)
class DaoStudent(DaoFunc):
    pass


@DB
class Database(DBFunc):
    def __init__(self):
        self.dao_student = DaoStudent()


@DB
class DatabaseStaticDao:
    dao_student = DaoStudent()


@DB
class DatabaseMix:
    dao_student1 = DaoStudent()

    def __init__(self):
        self.dao_student2 = DaoStudent()


# ====================================================================================================================
def test_parse_dao_list():
    db = DatabaseMix()
    assert len(db.dao_list) == 2
    assert db.dao_list[0] is not db.dao_list[1]


def test_multiple_parse_db():
    db1 = Database()
    db2 = Database()
    db3 = DatabaseStaticDao()
    db4 = DatabaseStaticDao()

    assert db1.dao_student is not db2.dao_student
    assert db3.dao_student is not db4.dao_student


@use_db(Database)
def test_create_tables(db):
    """建库"""
    db.create_tables()


@use_db(Database)
def test_create_with_comment(db):
    """注释功能"""
    db.dao_student.create_table()


@use_db(DatabaseStaticDao)
def test_static_dao_create_tables(db):
    """建库"""
    db.create_tables()


@use_db(DatabaseStaticDao)
def test_static_dao_create_with_comment(db):
    """注释功能"""
    db.dao_student.create_table()


if __name__ == '__main__':
    pytest.main(["-vv", "--capture=no", __file__])



# -*- coding:utf-8 -*-
from dataclasses import dataclass

import pytest

from sqlink import Entity, Define as d, Dao, DaoFunc
from sqlink.database import DB
from test.test_env import use_db


@Entity
@dataclass
class Student:
    name: str
    age: int
    phone: int = d.not_null
    weight: float = 50.0, d.not_null
    height: float = d.not_null, d.unique
    address: str = 'hit', d.not_null, d.unique, d.datatype("varchar(200)")
    student_id: int = d.auto_primary_key
    if_aged: bool = False


@Dao(Student)
class DaoStudent(DaoFunc):
    pass


@DB
class Database:
    dao_student = DaoStudent()


# ====================================================================================================================
@use_db(Database)
def test_create_tables(db):
    """建库"""
    db.create_tables()


@use_db(Database)
def test_create(db):
    db.dao_student.create_table()


if __name__ == '__main__':
    pytest.main(["-vv", "--capture=no", __file__])



# -*- coding:utf-8 -*-
from dataclasses import dataclass

import pytest

from sqlink import Entity, Define as d, Dao, DaoFunc
from sqlink.database import DB
from test.test_env import use_db


@Entity
@dataclass
class Student_Comment:
    name: str = d.comment('学生姓名')
    contact_info: float = d.comment('联系方式')
    time: str = d.comment('时间')
    abstract: str = d.comment('摘要')
    student_id: int = d.auto_primary_key, d.comment('学生id')


@Dao(Student_Comment)
class DaoStudent(DaoFunc):
    pass


@DB
class Database:
    dao_student = DaoStudent()


# ====================================================================================================================
@use_db(Database)
def test_create_tables(db):
    """建库"""
    db.create_tables()


@use_db(Database)
def test_create_with_comment(db):
    """注释功能"""
    db.dao_student.create_table()


if __name__ == '__main__':
    pytest.main(["-vv", "--capture=no", __file__])



# -*- coding:utf-8 -*-
from dataclasses import asdict, dataclass

import pytest

from sqlink import Entity, Define as d, Dao, DaoFunc, Sql
from sqlink.database import DB
from test.test_env import use_db


@Entity(check_type=True)
class Student:
    name: str
    age: int
    phone: int = d.not_null
    weight: float = 50.0, d.not_null
    height: float = d.not_null, d.unique
    address: str = 'hit', d.not_null, d.unique, d.datatype("varchar(200)")
    student_id: int = d.primary_key
    if_aged: bool = False


@Dao(entity=Student, fetch_type=dataclass)
class DaoStudent(DaoFunc):
    @Sql("select * from student where student_id=?;", fetch_type=Student)
    def select_student_by_id(self, student_id):
        pass

    @Sql("delete from student where student_id=?;")
    def delete_student_by_id(self, student_id):
        pass

    @Sql("select * from __ where student_id=?;", fetch_one=True, fetch_type=Student)
    def select_student_by_id_use_fetch_one(self):
        pass

    @Sql("select * from student where student_id=? limit 1;", fetch_type=Student)
    def select_student_by_id_limit1(self, student_id):
        pass


@DB
class Database:
    dao_student = DaoStudent()


# ====================================================================================================================
def generate_students(start, end):
    return [
        Student(name=f'李华{i}', age=i, phone=123456789,
                weight=50.0, height=100.0 + i, address=f'hit{i}',
                student_id=i)
        for i in range(start, end)
    ]


def insert_data(db: Database, num=100):
    students = generate_students(0, num)
    db.dao_student.insert(students)
    db.commit()
    return students


# ====================================================================================================================
@use_db(Database)
def test_fetchone(db: Database):
    """查询单记录"""
    students = insert_data(db)
    for student in students:
        select_id = student.student_id
        result1 = db.dao_student.select_student_by_id(select_id)
        result2 = db.dao_student.select_student_by_id_limit1(select_id)
        assert isinstance(result1, list)
        assert asdict(result1[0]) == asdict(result2)


@use_db(Database)
def test_fetchone_no_record(db: Database):
    """查询不存在的单记录"""
    students = insert_data(db)
    for student in students:
        select_id = student.student_id
        db.dao_student.delete_student_by_id(student_id=select_id)
        result1 = db.dao_student.select_student_by_id(select_id)
        result2 = db.dao_student.select_student_by_id_limit1(select_id)
        assert not result1
        assert result2 is None


@use_db(Database)
def test_sql_fetch_one(db: Database):
    students = insert_data(db)
    for student in students:
        select_id = student.student_id
        result1 = db.dao_student.select_student_by_id(select_id)
        result2 = db.dao_student.select_student_by_id_use_fetch_one(select_id)
        assert type(result1) == list
        assert type(result2) == Student
        assert result1[0] == result2


if __name__ == '__main__':
    pytest.main(["-vv", "--capture=no", __file__])



# -*- coding:utf-8 -*-
import sqlite3
from dataclasses import dataclass
from typing import Optional

import pytest

from sqlink import Entity, Define as d, Dao, DaoFunc, Insert
from sqlink.dao import Conflict, Sql, DB_TYPE_SQLITE, DB_TYPE_MYSQL
from sqlink.database import DB
from test.test_env import use_db


@Entity(check_type=True)
@dataclass
class Student:
    name: str = d.unique, d.datatype("varchar(200)")
    age: int = None
    id: int = d.auto_primary_key


@Dao(Student, fetch_type=Student)
class DaoStudent(DaoFunc):

    @Insert(conflict=Conflict.error)
    def insert_error(self):
        pass

    @Insert(conflict=Conflict.ignore)
    def insert_ignore(self):
        pass

    @Insert(conflict=Conflict.replace)
    def insert_replace(self):
        pass

    @Sql("select * from __ where name=? limit 1;")
    def get_student_by_name(self, name) -> Optional[Student]:
        pass


@DB
class Database:
    dao_student = DaoStudent()


students = [Student(name='张三', age=10),
            Student(name='张三', age=11),
            Student(name='李四', age=12)]


# ====================================================================================================================

@use_db(Database)
def test_insert_default(db: Database):
    if db.db_type == DB_TYPE_SQLITE:
        with pytest.raises(sqlite3.IntegrityError) as exc_info:
            for student in students:
                db.dao_student.insert(student)
        assert "UNIQUE constraint failed" in str(exc_info.value)
    elif db.db_type == DB_TYPE_MYSQL:
        import pymysql

        with pytest.raises(pymysql.err.IntegrityError) as exc_info:
            for student in students:
                db.dao_student.insert(student)
        assert "Duplicate entry" in str(exc_info.value)
        assert "for key 'student.name'" in str(exc_info.value)
    db.rollback()


@use_db(Database)
def test_insert_error(db: Database):
    if db.db_type == DB_TYPE_SQLITE:
        with pytest.raises(sqlite3.IntegrityError) as exc_info:
            for student in students:
                db.dao_student.insert_error(student)
        assert "UNIQUE constraint failed" in str(exc_info.value)
    elif db.db_type == DB_TYPE_MYSQL:
        import pymysql

        with pytest.raises(pymysql.err.IntegrityError) as exc_info:
            for student in students:
                db.dao_student.insert_error(student)
        assert "Duplicate entry" in str(exc_info.value)
        assert "for key 'student.name'" in str(exc_info.value)
    db.rollback()


@use_db(Database)
def test_insert_ignore(db: Database):
    for student in students:
        db.dao_student.insert_ignore(student)
    db.commit()

    select_student = students[0]
    student_conflict = db.dao_student.get_student_by_name(name=select_student.name)
    assert student_conflict.age == select_student.age


@use_db(Database)
def test_insert_replace(db: Database):
    for student in students:
        db.dao_student.insert_replace(student)
    db.commit()

    select_student = students[0]
    student_conflict = db.dao_student.get_student_by_name(name=select_student.name)
    assert student_conflict.age == students[1].age


@use_db(Database)
def test_insert_replace_on_primary_key(db: Database):
    students = [Student(name='张三', age=10, id=1),
                Student(name='张三', age=11, id=1)]
    for student in students:
        db.dao_student.insert_replace(student)
    db.commit()

    select_student = students[0]
    student_conflict = db.dao_student.get_student_by_name(name=select_student.name)
    assert student_conflict.age == students[1].age


if __name__ == '__main__':
    pytest.main(["-vv", "--capture=no", __file__])



# -*- coding:utf-8 -*-
import time
from collections import namedtuple
from dataclasses import asdict, dataclass

import pytest

from sqlink import Entity, Define as d, Dao, DaoFunc, Sql
from sqlink.database import DB
from test.test_env import use_db


@Entity(check_type=True)
@dataclass
class Student:
    name: str
    age: int
    phone: int = d.not_null
    weight: float = 50.0, d.not_null
    height: float = d.not_null, d.unique
    address: str = 'hit', d.not_null, d.unique, d.datatype("varchar(200)")
    student_id: int = d.primary_key
    if_aged: bool = False


@Dao(Student, fetch_type=Student)
class DaoStudent(DaoFunc):
    @Sql("select * from student;")
    def select_all(self):
        pass

    @Sql("select name, age,phone,weight, height,address,student_id,if_aged from student;", fetch_type=Student)
    def select_all_full(self):
        pass

    @Sql("select name, age,phone,weight, height,address,student_id from student;", fetch_type=Student)
    def select_all_part(self):
        pass

    @Sql("select age,name, student_id,weight, height,if_aged,address,phone from student;", fetch_type=Student)
    def select_all_full_no_order(self):
        pass

    @Sql("select age,name, student_id,weight, height,address,phone from student;", fetch_type=Student)
    def select_all_part_and_no_order(self):
        pass

    @Sql("select * from student;", fetch_type=namedtuple)
    def select_all_namedtuple(self):
        pass

    @Sql("select * from student where student_id=?;", fetch_type=Student)
    def select_student_by_id(self, student_id):
        pass

    @Sql("select * from student where student_id=? limit 1;", fetch_type=Student)
    def select_student_by_id_limit1(self, student_id):
        pass


@Dao(entity=Student, fetch_type=dict)
class DaoStudentDict(DaoFunc):
    @Sql("select * from student;")
    def select_all(self):
        pass

    @Sql("select * from student where student_id=? limit 1;")
    def select_student_by_id(self, student_id):
        pass

    @Sql("select * from student where student_id=?;")
    def select_student_by_id2(self, student_id):
        pass


@Dao(entity=Student, fetch_type=tuple)
class DaoStudentTuple(DaoFunc):
    @Sql("select * from student;")
    def select_all(self):
        pass

    @Sql("select * from student where student_id=? limit 1;")
    def select_student_by_id(self, student_id):
        pass

    @Sql("select * from student where student_id=?;")
    def select_student_by_id2(self, student_id):
        pass


@DB
class Database:
    dao_student = DaoStudent()
    dao_student_dict = DaoStudentDict()
    dao_student_tuple = DaoStudentTuple()


# ====================================================================================================================
def generate_students(start, end):
    return [
        Student(name=f'李华{i}', age=i, phone=123456789,
                weight=50.0, height=100.0 + i, address=f'hit{i}',
                student_id=i)
        for i in range(start, end)
    ]


def insert_data(db: Database, num=100):
    students = generate_students(0, num)
    db.dao_student.insert(students)
    db.commit()
    return students


# ====================================================================================================================
@use_db(Database)
def test_row_factory(db: Database):
    start = time.time()
    students = generate_students(0, 100000)
    print(f'\n生成耗时：{time.time() - start}')

    start = time.time()
    db.dao_student.insert(students)
    db.commit()
    end = time.time() - start
    print(f'插入耗时：{end}')

    # 默认数据类
    start = time.time()
    results2 = db.dao_student.select_all()
    end = time.time() - start
    print(f'entity_auto耗费时间：{end}, 查询结果数量；{len(results2)}')

    start = time.time()
    results3 = db.dao_student.select_all_full()
    end = time.time() - start
    print(f'entity_full_fast耗费时间：{end}, 查询结果数量；{len(results3)}')

    start = time.time()
    results4 = db.dao_student.select_all_part()
    end = time.time() - start
    print(f'entity_part耗费时间：{end}, 查询结果数量；{len(results4)}')

    start = time.time()
    results5 = db.dao_student.select_all_full_no_order()
    end = time.time() - start
    print(f'entity_full_not_order耗费时间：{end}, 查询结果数量；{len(results5)}')

    start = time.time()
    result6 = db.dao_student.select_all_part_and_no_order()
    end = time.time() - start
    print(f'entity_part_and_no_order耗费时间：{end}, 查询结果数量；{len(results4)}')
    assert results4 == result6
    assert results2 == results3 == results5

    start = time.time()
    results_dataclass = db.dao_student.select_all()
    end = time.time() - start
    print(f'dataclass耗费时间：{end}, 查询结果数量；{len(results_dataclass)}')

    start = time.time()
    results_dict = db.dao_student_dict.select_all()
    end = time.time() - start
    print(f'dict耗费时间：{end}, 查询结果数量；{len(results_dict)}')

    start = time.time()
    results_tuple = db.dao_student_tuple.select_all()
    end = time.time() - start
    print(f'tuple耗费时间：{end}, 查询结果数量；{len(results_tuple)}')

    start = time.time()
    results_namedtuple = db.dao_student.select_all_namedtuple()
    end = time.time() - start
    print(f'namedtuple耗费时间：{end}, 查询结果数量；{len(results_namedtuple)}')

    for item_dataclass, item_dict, item_tuple, item_namedtuple in zip(results_dataclass,
                                                                      results_dict,
                                                                      results_tuple,
                                                                      results_namedtuple):
        assert item_dict == item_namedtuple._asdict()
        assert item_tuple == item_namedtuple
        assert item_namedtuple._asdict() == asdict(item_dataclass)


@use_db(Database)
def test_dao_return_type_default_list(db: Database):
    students = insert_data(db)
    for student in students:
        result = db.dao_student.select_student_by_id(student.student_id)
        assert result
        assert isinstance(result[0], Student)
        assert result[0] == student


@use_db(Database)
def test_dao_return_type_default(db: Database):
    students = insert_data(db)
    for student in students:
        result = db.dao_student.select_student_by_id_limit1(student.student_id)
        assert isinstance(result, Student)
        assert result == student


@use_db(Database)
def test_dao_return_type_dict_list(db: Database):
    students = insert_data(db)
    for student in students:
        result = db.dao_student_dict.select_student_by_id2(student.student_id)
        assert result
        assert isinstance(result[0], dict)
        assert result[0] == asdict(student)


@use_db(Database)
def test_dao_return_type_dict(db: Database):
    students = insert_data(db)
    for student in students:
        result = db.dao_student_dict.select_student_by_id(student.student_id)
        assert isinstance(result, dict)
        assert result == asdict(student)


@use_db(Database)
def test_dao_return_type_tuple_list(db: Database):
    students = insert_data(db)
    for student in students:
        result = db.dao_student_tuple.select_student_by_id2(student.student_id)
        assert result
        assert isinstance(result[0], tuple)
        assert result[0] == tuple(asdict(student).values())


@use_db(Database)
def test_dao_return_type_tuple(db: Database):
    students = insert_data(db)
    for student in students:
        result = db.dao_student_tuple.select_student_by_id(student.student_id)
        assert isinstance(result, tuple)
        assert result == tuple(asdict(student).values())


if __name__ == '__main__':
    pytest.main(["-vv", "--capture=no", __file__])



# -*- coding:utf-8 -*-
from dataclasses import dataclass

import pytest

from sqlink import Entity, Define as d, Dao, DaoFunc, Sql
from sqlink.database import DB
from test.test_env import use_db


@Entity(check_type=True)
class Student:
    name: str
    age: int
    phone: int = d.not_null
    weight: float = 50.0, d.not_null
    height: float = d.not_null, d.unique
    address: str = 'hit', d.not_null, d.unique, d.datatype("varchar(200)")
    student_id: int = d.primary_key
    if_aged: bool = False


@Dao(Student, fetch_type=Student)
class DaoStudent(DaoFunc):

    @Sql("select if_aged from __ limit 1;", fetch_type=bool)
    def select_bool(self):
        pass

    @Sql("select if_aged from __ limit 1;")
    def select_bool_no_return_bool(self):
        pass


@DB
class Database:
    dao_student = DaoStudent()


# ====================================================================================================================
def generate_students(start, end):
    return [
        Student(name=f'李华{i}', age=i, phone=123456789,
                weight=50.0, height=100.0 + i, address=f'hit{i}',
                student_id=i)
        for i in range(start, end)
    ]


def insert_data(db: Database, num=100):
    students = generate_students(0, num)
    db.dao_student.insert(students)
    db.commit()
    return students


# ====================================================================================================================
@use_db(Database)
def test_bool(db: Database):
    insert_data(db, 1000)
    result1 = db.dao_student.select_bool()
    result2 = db.dao_student.select_bool_no_return_bool()
    assert type(result1) == bool
    assert type(result2.if_aged) == int


if __name__ == '__main__':
    pytest.main(["-vv", "--capture=no", __file__])



# -*- coding:utf-8 -*-
from dataclasses import dataclass, asdict

import pytest

from sqlink import Entity, Define as d, Dao, DaoFunc, Sql
from sqlink.database import DB
from test.test_env import use_db


@Entity(check_type=True)
@dataclass
class Student:
    name: str
    age: int
    phone: int = d.not_null
    weight: float = 50.0, d.not_null
    height: float = d.not_null, d.unique
    address: str = 'hit', d.not_null, d.unique, d.datatype("varchar(200)")
    student_id: int = d.primary_key
    if_aged: bool = False


@Dao(entity=Student, fetch_type=Student)
class DaoStudent(DaoFunc):
    @Sql("select * from student where student_id=?;", fetch_type=Student)
    def select_student_by_id(self, student_id):
        pass

    @Sql("select * from __;")
    def select_all_with_substitute(self):
        pass

    @Sql("update __ set name=? where student_id=?;")
    def update_name_by_id_with_substitute(self, name, student_id):
        pass

    @Sql("SELECT * FROM __ WHERE student_id = (SELECT MAX(student_id) FROM __);")
    def select_last_student_with_substitute(self):
        pass


@DB
class Database:
    dao_student = DaoStudent()


# ====================================================================================================================
def generate_students(start, end):
    return [
        Student(name=f'李华{i}', age=i, phone=123456789,
                weight=50.0, height=100.0 + i, address=f'hit{i}',
                student_id=i)
        for i in range(start, end)
    ]


def insert_data(db: Database, num=100):
    students = generate_students(0, num)
    db.dao_student.insert(students)
    db.commit()
    return students


# ====================================================================================================================
@use_db(Database)
def test_select_all_with_substitute(db):
    """使用表名替代符"""

    students = insert_data(db)
    results = db.dao_student.select_all_with_substitute()
    for result in results:
        assert asdict(result) == asdict(students[result.student_id])


@use_db(Database)
def test_update_name_with_substitute(db):
    """使用表名替代符更新"""
    students = insert_data(db)
    for student in students:
        select_id = student.student_id
        new_name = '好家伙'
        db.dao_student.update_name_by_id_with_substitute(name=new_name, student_id=select_id)
        db.commit()
        result = db.dao_student.select_student_by_id(select_id)
        assert result[0].name == new_name


@use_db(Database)
def test_multiple_table_substitute(db):
    """复杂sql中同时使用多个表名替代符"""
    students = insert_data(db)
    result = db.dao_student.select_last_student_with_substitute()
    assert result
    assert result[0].student_id == students[-1].student_id


if __name__ == '__main__':
    pytest.main(["-vv", "--capture=no", __file__])



# -*- coding:utf-8 -*-
from dataclasses import asdict, dataclass

import pytest

from sqlink import Entity, Define as d, Dao, Insert
from sqlink.database import DB
from test.test_env import use_db


@Entity(check_type=True)
@dataclass
class Student:
    name: str
    age: int
    phone: int = d.not_null
    weight: float = 50.0, d.not_null
    height: float = d.not_null, d.unique
    address: str = 'hit', d.not_null, d.unique, d.datatype("varchar(200)")
    student_id: int = d.primary_key
    if_aged: bool = False


@Dao(entity=Student, fetch_type=dataclass)
class DaoStudent:
    """不使用DaoFund类"""

    @Insert
    def insert(self, entity):
        pass

    def select_one_student(self, student_id: int):
        sql = "select * from student where student_id=?;"
        return self.execute(sql=sql, args=[student_id])

    def select_one_student_with_substitute(self, student_id: int):
        sql = "select * from __ where student_id=? limit 1;"
        return self.execute(sql=sql, args=[student_id])

    def select_one_student_with_limit(self, student_id: int):
        sql = "select * from student where student_id=? limit 1;"
        return self.execute(sql=sql, args=[student_id])

    def select_one_student_with_substitute_and_limit(self, student_id: int):
        sql = "select * from __ where student_id=? limit 1;"
        return self.execute(sql=sql, args=[student_id])

    def select_student_with_conditions(self, student_id: int, age: int):
        sql = "select * from student where student_id=? and age=? limit 1;"
        return self.execute(sql=sql, args=[student_id, age])


@DB
class Database:
    dao_student = DaoStudent()


# ====================================================================================================================
def generate_students(start, end):
    return [
        Student(name=f'李华{i}', age=i, phone=123456789,
                weight=50.0, height=100.0 + i, address=f'hit{i}',
                student_id=i)
        for i in range(start, end)
    ]


def insert_data(db: Database, num=100):
    students = generate_students(0, num)
    db.dao_student.insert(students)
    db.commit()
    return students


# ====================================================================================================================
@use_db(Database)
def test_execute_sql_without_dao_func(db: Database):
    """执行sql方法"""
    students = insert_data(db)
    for student in students:
        select_id = student.student_id
        result = db.dao_student.select_one_student(student_id=select_id)
        assert isinstance(result, list)
        assert result
        assert asdict(result[0]) == asdict(students[select_id])


@use_db(Database)
def test_execute_sql_with_substitute_without_dao_func(db: Database):
    """执行sql方法，使用替代符"""
    students = insert_data(db)
    for student in students:
        select_id = student.student_id
        result = db.dao_student.select_one_student_with_substitute(student_id=select_id)
        assert asdict(result) == asdict(students[select_id])


@use_db(Database)
def test_execute_sql_with_limit_without_dao_func(db: Database):
    """执行sql方法，使用limit"""
    students = insert_data(db)
    for student in students:
        select_id = student.student_id
        result = db.dao_student.select_one_student_with_limit(student_id=select_id)
        assert asdict(result) == asdict(students[select_id])


@use_db(Database)
def test_execute_sql_with_substitute_and_limit_without_dao_func(db: Database):
    """执行sql方法，使用替代符和limit"""
    students = insert_data(db)
    for student in students:
        select_id = student.student_id
        result = db.dao_student.select_one_student_with_substitute_and_limit(student_id=select_id)
        assert asdict(result) == asdict(students[select_id])


@use_db(Database)
def test_execute_sql_with_conditions_without_dao_func(db: Database):
    """执行sql方法，使用替代符和limit"""
    students = insert_data(db)
    for student in students:
        select_id = student.student_id
        result = db.dao_student.select_student_with_conditions(student_id=select_id, age=select_id)
        assert asdict(result) == asdict(students[select_id])


if __name__ == '__main__':
    pytest.main(["-vv", "--capture=no", __file__])



# -*- coding: utf-8 -*-
import copy
import os

from urllib.parse import urlparse, parse_qs


def DB(cls):
    """数据库类的装饰器。

    # 推荐用法
    Example1:
        @DB
        class Database(DBFunc):
            dao1 = DaoStudent()
            dao2 = ...
            dao3 = ...

    # 完整示例
    Example2:
        @Entity
        class Student:  # 定义一个数据类
            name: str
            student_id: int = Define.auto_primary_key

        @Dao(Student)
        class DaoStudent:  # 定义一个数据访问类

            @Sql("select * from student where student_id=?;")
            def get_student(self, student_id):
                pass

        @DB
        class Database:  # 定义一个数据库类，继承元数据库类
            dao1 = DaoStudent()  # 将各个数据访问类实例化为类变量，集中管理，统一对外。
            dao2 = ...
            dao3 = ...


        db = Database()  # 实例化数据库类，并传入数据库路径
        db.connect('sqlite://test.db')  # 连接数据库
        db.create_tables()  # 创建数据表

    Example3:
        @DB
        class Database:
            def __init__(self):
                self.dao1 = DaoStudent()  # 将各个数据访问类实例化为对象属性
                self.dao2 = ...
                self.dao3 = ...
    """

    class Wrapped(cls, DBFunctions):
        def __init__(self, *args, **kwargs):
            super().__init__(*args, **kwargs)
            params = cls.__dict__ | self.__dict__
            self.dao_list = []
            for attr, value in params.items():
                if hasattr(value, "entity") and hasattr(value, "update_cursor"):
                    new_value = copy.deepcopy(value)
                    setattr(self, attr, new_value)
                    self.dao_list.append(new_value)

    return Wrapped


class DBFunc:
    """由于IDE不能提示装饰器给出的方法，所以可以通过将数据库类继承此接口来获得代码提示。不继承不会造成任何影响，但推荐继承"""

    def connect(self, connect_string: str, *args, **kwargs):
        """通过标准的数据库连接串建立数据库连接，如sqlite数据库的 sqlite://test.db
        和mysql数据库的 mysql://user:password@localhost:3306/test_db """

    def disconnect(self, *args, **kwargs):
        """断开数据库连接"""

    def execute(self, sql: str, args=None, **kwargs):
        """给外部提供的直接执行sql的接口，避免了再调用内部的connection或者cursor"""

    def commit(self, *args, **kwargs):
        """提交事务"""

    def rollback(self, *args, **kwargs):
        """回滚事务"""

    def create_tables(self):
        """当表不存在时，才会创建数据表。因此可以反复调用该方法，而不会产生错误。"""

    def get_create_table_sql(self):
        """返回各个entity的建表sql语句"""


class DBFunctions(DBFunc):
    """元数据库类，提供库级的操作。"""

    def __init__(self, *args, **kwargs):
        self.connection = None
        self.cursor = None
        self.db_type = None
        self.dao_list = None

    def connect(self, connect_string: str, *args, **kwargs):
        result = parse_connection_string(connect_string)
        db_type = result['db_type']
        if db_type == 'sqlite':
            import sqlite3

            self.update_db_type('sqlite')
            self.connection = sqlite3.connect(os.path.normpath(result['database']), *args, **kwargs)
        elif db_type == 'mysql':
            import pymysql

            self.update_db_type('mysql')
            self.connection = pymysql.connect(user=result['username'],
                                              password=result['password'],
                                              host=result['host'],
                                              port=result['port'])
        else:
            raise ValueError(
                "暂不支持的数据库类型"
            )
        self.cursor = self.connection.cursor()
        for dao in self.dao_list:
            dao.update_cursor(cursor=self.cursor)

    def update_db_type(self, db_type: str):
        self.db_type = db_type
        for dao in self.dao_list:
            dao.update_db_type(db_type)

    def disconnect(self):
        if self.cursor:
            self.cursor.close()
        if self.connection:
            self.connection.close()

    def create_tables(self):
        for dao in self.dao_list:
            dao.create_table()  # noqa
        self.commit()

    def get_create_table_sql(self) -> list[str]:
        return [dao.get_create_table_sql() for dao in self.dao_list]

    def commit(self):
        self.connection.commit()

    def rollback(self):
        self.connection.rollback()

    def execute(self, sql: str, args=None, **kwargs):
        if args:
            self.cursor.execute(sql, args, **kwargs)
        else:
            self.cursor.execute(sql, **kwargs)


def parse_connection_string(string):
    result = {}
    parsed_url = urlparse(string)

    if parsed_url.scheme == 'mysql':
        result['db_type'] = 'mysql'
        result['username'] = parsed_url.username
        result['password'] = parsed_url.password
        result['host'] = parsed_url.hostname
        result['port'] = parsed_url.port
        result['options'] = parse_qs(parsed_url.query)
        result['database'] = os.path.basename(parsed_url.path)
    elif parsed_url.scheme == 'sqlite':
        result['db_type'] = 'sqlite'
    else:
        raise ValueError("数据库连接串格式错误")

    if parsed_url.scheme == 'sqlite':
        if parsed_url.netloc:
            result['database'] = os.path.normpath(parsed_url.netloc + parsed_url.path)
        else:
            result['database'] = os.path.normpath(parsed_url.path)

    return result



# -*- coding:utf-8 -*-
from dataclasses import dataclass

import pytest

from sqlink import Entity, Define as d, Dao, DaoFunc, Sql
from sqlink.database import DB
from test.test_env import use_db


@Entity(check_type=True)
@dataclass
class Student:
    name: str
    age: int
    student_id: int = d.primary_key


@Dao(Student, fetch_type=Student)
class DaoStudent(DaoFunc):
    @Sql("select name from __;")
    def select_name_by_id_without_single_field_list(self):
        pass

    @Sql("select name from __ where student_id=? limit 1;")
    def select_name_by_id_without_single_field(self, student_id: int):
        pass

    @Sql("select name from __;", fetch_type=str)
    def select_name_by_id_with_single_field_list(self):
        pass

    @Sql("select name from __ where student_id=? limit 1;", fetch_type=str)
    def select_name_by_id_with_single_field(self, student_id: int):
        pass


@DB
class Database:
    dao_student = DaoStudent()


# ====================================================================================================================
def generate_students(start, end):
    return [
        Student(name=f'李华{i}', age=i, student_id=i)
        for i in range(start, end)
    ]


def insert_data(db: Database, num=100):
    students = generate_students(1, num)
    db.dao_student.insert(students)
    db.commit()
    return students


# ====================================================================================================================
@use_db(Database)
def test_select_one_field(db: Database):
    students = insert_data(db)
    for student in students:
        s1 = db.dao_student.select_name_by_id_without_single_field(student.student_id)
        s2 = db.dao_student.select_name_by_id_with_single_field(student.student_id)
        assert s1.name == student.name
        assert s2 == student.name

        result1 = db.dao_student.select_name_by_id_without_single_field_list()
        result2 = db.dao_student.select_name_by_id_with_single_field_list()
        for item1, item2 in zip(result1, result2):
            assert type(item1.name) == str
            assert type(item2) == str


if __name__ == '__main__':
    pytest.main(["-vv", "--capture=no", __file__])
